function [output] = CalculateExtendedTransition(u, v, itQty, scalesQty, interpMethod, version)

% Compute scalesQty scales of Gaussian pyramid (up to 8 pixels)
uCell = cell(1,scalesQty);
uCell{1} = u;
for i=1:(scalesQty-1)
    uCell{i+1} = pyramid(uCell{i});
end

vCell = cell(1,scalesQty);
vCell{1} = v;
for i=1:(scalesQty-1)
    vCell{i+1} = pyramid(vCell{i});
end

% Estimate the shift in a multi-scale fashion
acumShift = [0;0];
err = false;
for i=scalesQty:-1:2
    shift = CalculateTransition(uCell{i}, vCell{i}, itQty, interpMethod, false);
    if (sum(isnan(shift)) == 0)
        acumShift = acumShift .* 2 + shift .* 2;
        [vCell{i-1}, x, y] = IntegerShift(vCell{i-1}, acumShift(1), acumShift(2));
        if (x >= size(vCell{i-1},2) || y >= size(uCell{i-1},1))
            err = true;
            output = [0;0];
            break;
        end
        if (i-1 == 1)
            acumShift = [x;y];
        end
    else
        break;
    end
end
if (~err)
    % Finally, on the original scale, perform shift estimation using the
    % full selected version of the algorithm
    switch version
        case 'TLS'
            [dx, dy] = CalculateGradient(uCell{1});
            output = acumShift + CalculateTransitionBetweenFramesTLS(uCell{1},vCell{1}, dx,dy,itQty,interpMethod, true);
        case 'LS'
            [dx, dy] = CalculateGradient(uCell{1});
            output = acumShift + CalculateTransitionBetweenFramesTLS(uCell{1},vCell{1}, dx,dy,itQty,interpMethod, false);
        case 'Unbiased'
            % This uses the other simpler gradient estimation for
            % performance reasons
            [dx, dy] = CalculateGradient2(uCell{1});
            output = acumShift + CalculateUnbiasedTransitionBetweenFrames(uCell{1},vCell{1}, dx,dy);
    end
            
end
