function [] = run_algo(inputFile, outputDir, noiseModel, a, b, x0, y0, x1, y1, dX, dY)
% First perform the simulation
a = str2num(a);
b = str2num(b);
x0 = uint16(str2double(x0)) + 1;
y0 = uint16(str2double(y0)) + 1;
x1 = uint16(str2double(x1)) + 1;
y1 = uint16(str2double(y1)) + 1;
dX = str2num(dX);
dY = str2num(dY);
switch (noiseModel)
    case '1'
        noiseModel = 'gaussian';
    case '2'
        noiseModel = 'pleiades';
    case '3'
        noiseModel = 'poisson';
    case '0'
        noiseModel = 'no';
end
        
fprintf('Subimage: (%d, %d), (%d,%d). Shift: (%f, %f)\n', x0,y0, x1, y1, dX,dY);
fprintf('Noise model:%s. Params (a=%f, b=%f)\n', noiseModel, a, b);



warning('off','images:initSize:adjustingMag');
warning('off', 'MATLAB:interp2:NaNstrip');

% Simulation params
topLeftPos = [y0,x0]; % Subimage top left position (width and height is fixed to 37 pixels)
visSurface = 1;

[qtyMethods,methods,~] = EvaluateAlgorithms([], [], [], [], 0, 0, 0);
% Read image
refImg = double(imread(inputFile));
%refImg = double(imread(img));
%refImg = impyramid(refImg, 'reduce');
%refImg = impyramid(refImg, 'reduce');
%refImg = impyramid(refImg, 'reduce');
   
imSize = [y1-y0, x1-x0];
%scalesQty = 1;

%fprintf('It. Qty: %d, scalesQty: %d, interpMethod: %s, thSNR: %d, Noise: %s ,minVisSurface: %f\n', itQty, scalesQty, interpMethod, thSNR, model, minVisSurface);
shift = [dY dX];
[originalImages] = SimulateDisplacement(refImg, topLeftPos, shift, imSize, visSurface, false);

SaveImage(originalImages(:,:,1), [outputDir, '/', 'im1.png']);
SaveImage(originalImages(:,:,2), [outputDir, '/', 'im2.png']);
%imwrite(uint8(round(originalImages(:,:,1))), [outputDir, '/', 'im1.png']);
%imwrite(uint8(round(originalImages(:,:,2))), [outputDir, '/', 'im2.png']);
if (~strcmp(noiseModel,'no'))
    [images] = SimulateNoise(originalImages, noiseModel, a,b, false);
else
    images = originalImages;
end
SaveImage(images(:,:,1), [outputDir, '/', 'im1N.png']);
SaveImage(images(:,:,2), [outputDir, '/', 'im2N.png']);
%imwrite(uint8(round(images(:,:,1))), [outputDir, '/', 'im1N.png']);
%imwrite(uint8(round(images(:,:,2))), [outputDir, '/', 'im2N.png']);

imagesVST = zeros(size(images));
noiselessImagesVST = zeros(size(images));
for i=1:2
    imagesVST(:,:,i) = ApplyVST(images(:,:,i), noiseModel, a, b);
    noiselessImagesVST(:,:,i) = ApplyVST(originalImages(:,:,i), noiseModel, a, b);
end

% Estimate the noise
[sigma2] = EstimateNoise(imagesVST);
[sigma2error] = EstimateNoise(noiselessImagesVST);
[sigma2NoVST] = EstimateNoise(images);
fprintf('RealSigma: %f - Sigma: %f - SigmaError: %f, Dif. between estimated sigma and real sigma: %f\n', a, sqrt(sigma2), sqrt(sigma2error),(sqrt(sigma2) - a));
[dxFDVST, dyFDVST] = CalculateGradient(imagesVST(:,:,1));
[dxFD, dyFD] = CalculateGradient(images(:,:,1));
[dxFDNoNoise, dyFDNoNoise] = CalculateGradient(originalImages(:,:,1));

[varX, varY, ~, eigenRatio] = CalculateCRLB(dxFDVST(1:end-1,1:end-1), dyFDVST(1:end-1,1:end-1), sigma2, true);
[varXNoVST, varYNoVST, ~, eigenRatioNoVST] = CalculateCRLB(dxFD(1:end-1,1:end-1), dyFD(1:end-1,1:end-1), sigma2NoVST, true);
[realVarXNoVST, realVarYNoVST, ~, realEigenRatioNoVST] = CalculateCRLB(dxFDNoNoise(1:end-1,1:end-1), dyFDNoNoise(1:end-1,1:end-1), a^2, false);
%[realVarX, realVarY, ~, realEigenRatio] = CalculateCRLB(noiselessImagesVST(:,:,1), a^2, false);
fprintf('CR (VST): %f, %f - CR (No VST): %f, %f - Real CR (no VST): %f, %f: \n', sqrt(varX), sqrt(varY), sqrt(varXNoVST), sqrt(varYNoVST), sqrt(realVarXNoVST), sqrt(realVarYNoVST));

% Now estimate the shifts
u = imagesVST(:,:,1);
v = imagesVST(:,:,2);
[~, ~, RMSEs] = EvaluateAlgorithms(u, v, originalImages(:,:,1), originalImages(:,:,2), shift, sigma2, a);


methodsFile = fopen([outputDir, '/methods.txt'], 'w');
%fprintf(methodsFile, 'methods = [');
RMSEFile = fopen([outputDir, '/RMSEs.txt'], 'w');
%fprintf(RMSEFile, 'RMSEs = [');
for aux=1:qtyMethods-1
    fprintf('Method: %s - RMSE: %.4f\n', methods{aux}, RMSEs(aux));
    fprintf(methodsFile, '%s\n', methods{aux});
    fprintf(RMSEFile, '%.4f\n', RMSEs(aux));
end
fprintf('Method: %s - RMSE: %.4f\n', methods{qtyMethods}, RMSEs(qtyMethods));
%fprintf(methodsFile, '"%s"]', methods{qtyMethods});
%fprintf(RMSEFile, '"%.4f"]', RMSEs(qtyMethods));
fclose(methodsFile);
fclose(RMSEFile);
%res = struct('a',a,'b',b,'shift',shift, 'topLeftY', topLeftPos(1),'topLeftX',topLeftPos(2),'sigma2',sigma2, 'CRLB', sqrt(sqrt(varX)^2 + sqrt(varY)^2), 'CRLBNoVST', sqrt(sqrt(varXNoVST)^2 + sqrt(varYNoVST)^2), 'realCRLBNoVST', sqrt(sqrt(realVarXNoVST)^2 + sqrt(realVarYNoVST)^2), 'eigenRatio', eigenRatio, 'eigenRatioNoVST', eigenRatioNoVST, 'realEigenRatio', realEigenRatioNoVST, 'avgError', acumError);
%results{k} = res;

end